; This is a rotary encoder to servo controler
; 21NOV14 V00.00 PJS New

#define	CODE_VER_STRING "Peter Shabino 04NOV14 enc2servo V00.00 www.wire2wire.org for tymkrs.com" ;Just in ROM

;****************************************************************************************
; port list
; Vss(14)
; Vdd(1)
; RA0(13)   ICSPDAT
; RA1(12)   ICSPCLK
; RA2(11)
; RA3(4)	Vpp/#MCLR
; RA4(3)
; RA5(2)    > servo out
; RC0(10)	< A input
; RC1(9)	< B input
; RC2(8)    < button input (centers the servo)
; RC3(7)
; RC4(6)
; RC5(5)
;****************************************************************************************


;-----------------------------------------------------------------------------------------------------------
; compiler configuration
;-----------------------------------------------------------------------------------------------------------
		; set processor type and output file type (intel merged hex (.hex))
		list	f=INHX8M
		; PIC16F1705 Configuration Bit Settings
        #include "p16F1705.inc"
        ; CONFIG1
        ; __config 0xCF84
         __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_ON & _MCLRE_OFF & _CP_OFF & _BOREN_ON & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
        ; CONFIG2
        ; __config 0xD7FF
         __CONFIG _CONFIG2, _WRT_OFF & _PPS1WAY_ON & _ZCDDIS_ON & _PLLEN_OFF & _STVREN_ON & _BORV_LO & _LPBOR_ON & _LVP_OFF


;-----------------------------------------------------------------------------------------------------------
;constants
;-----------------------------------------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------------------------
;variables (0x20 - 0x6F only), 0x0A0-0x0EF, 0x120-0x16f, 0x1A0-0x1EF, 0x1A0-0x1EF, 0x220-0x26F, 0x2A0-0x2EF, 0x320-0x32F
;-----------------------------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------------------------------------
;global variables (0x70 - 0x7F only)
;-----------------------------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------------------------------------
;put the following at address 0000h (starting vector)
;-----------------------------------------------------------------------------------------------------------
		org     0000h
		goto	START

;-----------------------------------------------------------------------------------------------------------
;put the following at address 0004h (IRQ vector)
;-----------------------------------------------------------------------------------------------------------
		org     0004h
		goto	ENDLOOP     ; since not used go to the lockout loop

;-----------------------------------------------------------------------------------------------------------
;put the following at address 0005h (Start of program space)
;-----------------------------------------------------------------------------------------------------------
		org     0005h
START
        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'00'
        ;-----------------
        clrf    PORTA
        clrf    PORTC
        ; set up timer 2 to max period ~ 61Hz
        movlw   0xFF        ; max period
        movwf   PR2
        movlw   0x07        ; 1:1 post scalar, timer on, 1:64 pre scalar
        movwf   T2CON

        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'01'
        ;-----------------
		; set up IOs
		movlw	0xDF		; all pins inputs other than RA5 as a output for PWM
		movwf	TRISA
		movlw	0xFF		; all pins input
		movwf	TRISC

        ; set up clocks to 4MHz
        movlw   0x68
        movwf   OSCCON

        ; check if the high freq osc is stable
CHECKLA
        btfss   OSCSTAT, HFIOFL
        goto    CHECKLA
CHECKHA
        btfss   OSCSTAT, HFIOFS
        goto    CHECKHA


        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'03'
        ;-----------------
        ; only affects the ADC inputs not the other analog functions
        clrf    ANSELA      ; no analog
        clrf    ANSELC      ; no analog

        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'05'
        ;-----------------

        clrf    CCPTMRS     ; all PWM based off timer 2
        movlw   0x0c        ; 28 - 0c = max rotation of the servo 18 is the center
        movwf   CCPR1L      ; inital value to mid point
        movwf   CCPR1H      ; inital value to mid point
        movlw   0x0C        ; CCP1 on PWM mode
        movwf   CCP1CON


        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'28'
        ;-----------------
		; Unlock the peripheral pin selects required sequence, next 5 instructions
		movlw	0x55
		movwf	PPSLOCK
		movlw	0xAA
		movwf	PPSLOCK
		; clear PPSLOCKED bit to enable writes
		bcf		PPSLOCK, PPSLOCKED

        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'29'
        ;-----------------
        ; set up PWM1 output pin
		movlw	0x0C		; set up RA5 to the output of PWM1(CCP1)
		movwf	RA5PPS

        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'28'
        ;-----------------
		; Lock the peripheral pin selects required sequence, next 5 instructions
		movlw	0x55
		movwf	PPSLOCK
		movlw	0xAA
		movwf	PPSLOCK
		; Set PPSLOCKED bit to disable writes
		bsf		PPSLOCK, PPSLOCKED

        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'00'
        ;-----------------
 
MAINLOOP
        btfsc   PORTC, 2    ; check if the button was pressed and center the servo if so
        goto    MID
        btfsc   PORTC, 0    ; wait for the rotary encoder clock (A) to go low
        goto    MAINLOOP
WAITLOW
        btfsc   PORTC, 2    ; check if the button was pressed and center the servo if so
        goto    MID
        btfss   PORTC, 0    ; wait for the rotary encoder clock (A) to go high
        goto    WAITLOW

        btfsc   PORTC, 1    ; check rotary encoder direction (B) if high go to reverse
        goto    REV

        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'05'
        ;-----------------
        movlw   0x28          ; check if at the max value
        xorwf   CCPR1L, W     ; if at the max value status zero bit will be set
        btfss   STATUS, Z     ; if at the max value do not increment the value
        incf   CCPR1L, F      ; inital value to mid point
        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'00'
        ;-----------------

        goto    MAINLOOP


REV
        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'05'
        ;-----------------
        movlw   0x0C            ; check if at the min value
        xorwf   CCPR1L, W       ; if at the min value status zero bit will be set
        btfss   STATUS, Z       ; if at the min value do not decrement the value
        decf   CCPR1L, F        ; inital value to mid point
        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'00'
        ;-----------------

        goto    MAINLOOP

MID
        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'05'
        ;-----------------
        movlw  0x18
        movwf   CCPR1L      ; inital value to mid point
        ;-----------------
        ; change banks
        ;-----------------
        movlb   d'00'
        ;-----------------

        goto    MAINLOOP

ENDLOOP
        ;catchall loop if things go bad just stop.
        goto    ENDLOOP


		de  CODE_VER_STRING     ; put the description string in the ROM
		end


